//@description: Bumpy, fresnel-shiny, dielectric, textured, with two quadratic-falloff lights
//@tags: Bump fresnel-shine dielectric texture lights//@help:a phong-shaded metal-style surface
//@credits:copyright nvidia corporation

//transforms
float4x4 tWIT:WORLDINVERSETRANSPOSE;
float4x4 tWVP:WORLDVIEWPROJECTION;
float4x4 tW:WORLD;
float4x4 tVI:VIEWINVERSE;

//light1
float3 lPos1 <string uiname="Light1 Position";> ={0,0,-1};
float4 lCol1 <bool color=true;string uiname="Light1 Color";> =1;
float lInt1 <string uiname="Light1 Intensity";float uimin=1.0;float uimax=50.0;> =2;

//light2
float3 lPos2 <string uiname="Light2 Position";> ={-1,0,1};
float4 lCol2 <bool color=true;string uiname = "Light2 Color";> = {0.5,0.5,1.0,1.0};
float lInt2 <string uiname="Light2 Intensity";float uimin=1.0;float uimax=50.0;> =0.5;

//material properties
float4 AmbiCol <bool color=true;string uiname="Ambient Color";> ={0.07,0.07,0.07,1.0};
float4 SurfCol <bool color=true;string uiname="Surface Color";> ={1.0,1.0,1.0,1.0};
float Kd <string uiname="Diffuse";float uimin=0.0;float uimax=1.5;> =1;
float Ks <string uiname="Specular";float uimin=0.0;float uimax=1.5;> =1;
float SpecExpon <string uiname="Specular Power";float uimin=1.0;float uimax=128.0;> =12;
float Bumpy <string uiname="Bumpiness";float uimin=0.0;float uimax=10.0;> =1;
float Kr <string uiname="Reflection Max";float uimin=0.0;float uimax=1.5;> =1;
float KrMin <string uiname="Reflection Min";float uimin=0.0;float uimax=0.2;float uistep=0.001;> =0.05;
float FresExp <string uiname="Fresnel Exponent";float uimin=1.0;float uimax=7.0;> =5;

float4x4 tTex <bool uvspace=true; string uiname="Texture Transform";>;
float4x4 tCol <string uiname="Color Transform";>;

//samplers
Texture2D ColorMap;
Texture2D NormalMap;
Texture2D SpecularMap;

SamplerState linearSampler:IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

TextureCube EnvironmentMap;
SamplerState EnvironmentSampler:IMMUTABLE

{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
	AddressW = Clamp;
};

struct vs2ps
{
    float4 Pos:SV_POSITION;
	float4 TexCd:TEXCOORD0;
	float3 lDir1:TEXCOORD1;
	float3 WorldNormal:TEXCOORD2;
	float3 WorldPos:TEXCOORD3;
	float3 WorldEyePos:TEXCOORD4;
	float3 WorldTangent:TEXCOORD5;
	float3 WorldBinorm:TEXCOORD6;
	float3 lDir2:TEXCOORD7;
};

vs2ps VS(
	float4 PosO:POSITION,
	float4 UV:TEXCOORD0,
	float4 Normal:NORMAL,
	float4 Tangent:TANGENT,
	float4 Binormal:BINORMAL)
{
    vs2ps OUT=(vs2ps)0;
	OUT.WorldNormal=mul(Normal,tWIT).xyz;
	OUT.WorldTangent=mul(Tangent,tWIT).xyz;
	OUT.WorldBinorm=mul(Binormal,tWIT).xyz;
	
	float4 Po = float4(PosO.xyz,1);
	float3 Pw = mul(Po,tW).xyz;
	OUT.WorldPos = Pw;
	OUT.lDir1 = lPos1.xyz-Pw;
	OUT.lDir2 = lPos2.xyz-Pw;
	OUT.TexCd = mul(UV,tTex);
	OUT.WorldEyePos = tVI[3].xyz-Pw;
	OUT.Pos = mul(Po,tWVP);
	return OUT;
}

float4 PS(vs2ps IN): SV_Target
{
	float3 map=ColorMap.Sample(linearSampler,IN.TexCd.xy).rgb;
	float3 bumps=Bumpy*(NormalMap.Sample(linearSampler,IN.TexCd.xy).xyz*2-1);
	float gloss=Ks*SpecularMap.Sample(linearSampler,IN.TexCd.xy).x;
	
	float3 Nn=normalize(IN.WorldNormal);
	float3 Tn=normalize(IN.WorldTangent);
	float3 Bn=normalize(IN.WorldBinorm);
	float3 Nb=Nn+(bumps.x*Tn+bumps.y*Bn);
           Nb=normalize(Nb);
	
	float3 Vn = normalize(IN.WorldEyePos);
	
	//first light
	float falloff = lInt1/dot(IN.lDir1,IN.lDir1);
	float3 Ln=normalize(IN.lDir1);
	float3 Hn=normalize(Vn+Ln);
	float hdn = dot(Hn,Nb);
	float ldn = dot(Ln,Nb);
	float4 litV = lit(ldn,hdn,SpecExpon);
	float3 incident = falloff*litV.y*lCol1.rgb;
	float3 diffContrib = incident;
	float3 specContrib = litV.z*gloss*incident;

	//second light
	falloff = lInt2/dot(IN.lDir2,IN.lDir2);
	Ln = normalize(IN.lDir2);
	Hn = normalize(Vn+Ln);
	hdn = dot(Hn,Nb);
	ldn = dot(Ln,Nb);
	litV = lit(ldn,hdn,SpecExpon);
	incident = falloff*litV.y*lCol2.rgb;
	diffContrib += incident;
	specContrib += litV.z*gloss*incident;
	
	float3 reflVect = reflect(Vn,Nb);
	float vdn = dot(Vn,Nb);
	float fres = KrMin+(Kr-KrMin)*pow(abs(1-abs(vdn)),FresExp);
	float3 reflColor=fres*EnvironmentMap.Sample(EnvironmentSampler,-reflVect).rgb;
	float3 result=(SurfCol.rgb*map*(Kd*diffContrib+AmbiCol.rgb))+specContrib+reflColor;
	return mul((float4(result.rgb,1)),tCol);
}


technique10 BumpGlossed
{
	pass P0
	{
		SetVertexShader( CompileShader(vs_4_0,VS()));
		SetPixelShader( CompileShader(ps_4_0,PS()));
	}
}





