//@help:bumpy,fresnel-shiny,dielectric,textured surface with two quadratic-falloff lights
//@tags:bump,gloss,fresnel,material
//@credits:copyright nvidia corporation

//transforms
float4x4 tWIT:WORLDINVERSETRANSPOSE;
float4x4 tWVP:WORLDVIEWPROJECTION;
float4x4 tW:WORLD;
float4x4 tVI:VIEWINVERSE;

//light1
float3 lPos1 <string UIName="Light1 Position";> ={0,0,-1};
float4 lCol1:COLOR <string UIName="Light1 Color";> =1;
float lInt1 <string UIName="Light1 Intensity";float UIMin=1.0;float UIMax=50.0;> =2;
//light2
float3 lPos2 <string UIName="Light2 Position";> ={-1,0,1};
float4 lCol2:COLOR <string UIName = "Light2 Color";> = {0.5,0.5,1.0,1.0};
float lInt2 <string UIName="Light2 Intensity";float UIMin=1.0;float UIMax=50.0;> =0.5;
//light3
float3 lPos3 <string UIName="Light3 Position";> ={-1,0,1};
float4 lCol3:COLOR <string UIName = "Light3 Color";> = {0.5,0.5,1.0,1.0};
float lInt3 <string UIName="Light3 Intensity";float UIMin=1.0;float UIMax=50.0;> =0.5;
//light4
float3 lPos4 <string UIName="Light4 Position";> ={-1,0,1};
float4 lCol4:COLOR <string UIName = "Light4 Color";> = {0.5,0.5,1.0,1.0};
float lInt4 <string UIName="Light4 Intensity";float UIMin=1.0;float UIMax=50.0;> =0.5;


//material properties
float4 AmbiCol:COLOR <string UIName="Ambient Color";> ={0.07,0.07,0.07,1.0};
float4 SurfCol:COLOR <string UIName="Surface Color";> ={1.0,1.0,1.0,1.0};
float Kd <string UIName="Diffuse";float UIMin=0.0;float UIMax=1.5;> =1;
float Ks <string UIName="Specular";float UIMin=0.0;float UIMax=1.5;> =1;
float SpecExpon <string UIName="Specular Power";float UIMin=1.0;float UIMax=128.0;> =12;
float Bumpy <string UIName="Bumpiness";float UIMin=0.0;float UIMax=10.0;> =1;
float Kr <string UIName="Reflection Max";float UIMin=0.0;float UIMax=1.5;> =1;
float KrMin <string UIName="Reflection Min";float UIMin=0.0;float UIMax=0.2;float UIStep=0.001;> =0.05;
float FresExp <string UIName="Fresnel Exponent";float UIMin=1.0;float UIMax=7.0;> =5;

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tCol <string uiname="Color Transform";>;

//samplers
texture ColorMap;
sampler2D s0=sampler_state
{
	Texture=(ColorMap);
	MinFilter=Linear;
	MagFilter=Linear;
	MipFilter=Linear;
	AddressU=Wrap;
	AddressV=Wrap;
};

texture NormalMap;
sampler2D s1=sampler_state
{
	Texture=(NormalMap);
	MinFilter=Linear;
	MagFilter=Linear;
	MipFilter=Linear;
	AddressU=Wrap;
	AddressV=Wrap;
};

texture SpecularMap;
sampler2D s2=sampler_state
{
	Texture=(SpecularMap);
	MinFilter=Linear;
	MagFilter=Linear;
	MipFilter=Linear;
	AddressU=Wrap;
	AddressV=Wrap;
};

texture EnvironmentMap;
sampler s3=sampler_state
{
	Texture=(EnvironmentMap);
	MinFilter=Linear;
	MagFilter=Linear;
	MipFilter=Linear;
	AddressU=clamp;
	AddressV=clamp;
	AddressW=clamp;
};

struct vs2ps
{
	float4 Pos:POSITION;
	float4 TexCd:TEXCOORD0;
	float3 WorldNormal:TEXCOORD1;
	float3 WorldPos:TEXCOORD2;
	float3 WorldEyeVec:TEXCOORD3;
	float3 WorldTangent:TEXCOORD4;
	float3 WorldBinorm:TEXCOORD5;
	float3 lDir1:TEXCOORD6;
	float3 lDir2:TEXCOORD7;
	float3 lDir3:TEXCOORD8;
	float3 lDir4:TEXCOORD9;
};

vs2ps vs(
	float3 PosO:POSITION,
	float4 UV:TEXCOORD0,
	float4 NormO:NORMAL,
	float4 Tangent:TANGENT0,
	float4 Binorm:BINORMAL0)
{
	vs2ps OUT = (vs2ps)0;
	OUT.WorldNormal = mul(NormO,tWIT).xyz;
	OUT.WorldTangent = mul(Tangent,tWIT).xyz;
	OUT.WorldBinorm = mul(Binorm,tWIT).xyz;
	float4 Po = float4(PosO.xyz,1);
	float3 Pw = mul(Po,tW).xyz;
	OUT.WorldPos = Pw;
	
	OUT.lDir1 = lPos1.xyz-Pw;
	OUT.lDir2 = lPos2.xyz-Pw;
	OUT.lDir3 = lPos3.xyz-Pw;
	OUT.lDir4 = lPos4.xyz-Pw;
	
	
	OUT.TexCd = mul(UV,tTex);
	OUT.WorldEyeVec = tVI[3].xyz-Pw;
	OUT.Pos = mul(Po,tWVP);
	return OUT;
}

float4 ps(vs2ps IN):COLOR
{
	float3 map = tex2D(s0,IN.TexCd.xy).rgb;
	float3 bumps = Bumpy*(tex2D(s1,IN.TexCd.xy).xyz-(.5).xxx);
	float gloss = Ks*tex2D(s2,IN.TexCd.xy).x;
	float3 Nn = normalize(IN.WorldNormal);
	float3 Tn = normalize(IN.WorldTangent);
	float3 Bn = normalize(IN.WorldBinorm);
	float3 Nb = Nn+(bumps.x*Tn+bumps.y*Bn);
    	   Nb = normalize(Nb);
	float3 Vn = normalize(IN.WorldEyeVec);
	
	// first light
	float falloff = lInt1/dot(IN.lDir1,IN.lDir1);
	float3 Ln = normalize(IN.lDir1);
	float3 Hn = normalize(Vn+Ln);
	float hdn = dot(Hn,Nb);
	float ldn = dot(Ln,Nb);
	float4 litV = lit(ldn,hdn,SpecExpon);
	float3 incident = falloff*litV.y*lCol1.rgb;
	float3 diffContrib = incident;
	float3 specContrib = litV.z*gloss*incident;
	
	// second light
	falloff = lInt2/dot(IN.lDir2,IN.lDir2);
	Ln = normalize(IN.lDir2);
	Hn = normalize(Vn+Ln);
	hdn = dot(Hn,Nb);
	ldn = dot(Ln,Nb);
	litV = lit(ldn,hdn,SpecExpon);
	incident = falloff*litV.y*lCol2.rgb;
	diffContrib += incident;
	specContrib += litV.z*gloss*incident;
	
	// third light
	falloff = lInt3/dot(IN.lDir3,IN.lDir3);
	Ln = normalize(IN.lDir3);
	Hn = normalize(Vn+Ln);
	hdn = dot(Hn,Nb);
	ldn = dot(Ln,Nb);
	litV = lit(ldn,hdn,SpecExpon);
	incident = falloff*litV.y*lCol3.rgb;
	diffContrib += incident;
	specContrib += litV.z*gloss*incident;
	
	// fourth light
	falloff = lInt4/dot(IN.lDir4,IN.lDir4);
	Ln = normalize(IN.lDir4);
	Hn = normalize(Vn+Ln);
	hdn = dot(Hn,Nb);
	ldn = dot(Ln,Nb);
	litV = lit(ldn,hdn,SpecExpon);
	incident = falloff*litV.y*lCol4.rgb;
	diffContrib += incident;
	specContrib += litV.z*gloss*incident;
	
	float3 reflVect = reflect(Vn,Nb);
	float vdn = dot(Vn,Nb);
	float fres = KrMin+(Kr-KrMin)*pow(1-abs(vdn),FresExp);
	float3 reflColor = fres*texCUBE(s3,float4(-reflVect,1)).rgb;
	float3 result=(SurfCol.rgb*map*(Kd*diffContrib+AmbiCol.rgb))+specContrib+reflColor;
	return mul((float4(result.rgb,1)),tCol);
}

technique BumpGlossed 
{
	pass p0 
	{
		VertexShader=compile vs_3_0 vs();
		PixelShader=compile ps_3_0 ps();	
	}
}
